<?php

namespace Drupal\feeds_ical\Feeds\Parser;

use Drupal\feeds\Component\XmlParserTrait;
use Drupal\feeds\Exception\EmptyFeedException;
use Drupal\feeds\FeedInterface;
use Drupal\feeds\Plugin\Type\Parser\ParserInterface;
use Drupal\feeds\Plugin\Type\PluginBase;
use Drupal\feeds\Result\FetcherResultInterface;
use Drupal\feeds\Result\ParserResult;
use Drupal\feeds\StateInterface;
use Drupal\feeds_ical\Feeds\Item\IcalItem;
use ICal\ICal;

/**
 * Defines an Ical Feeds parser.
 *
 * @FeedsParser(
 *   id = "feeds_ical",
 *   title = @Translation("Ical Parser"),
 *   description = @Translation("Parse Ical Feed.")
 * )
 */
class IcalParser extends PluginBase implements ParserInterface {
  use XmlParserTrait;

  /**
   * {@inheritdoc}
   */
  public function parse(FeedInterface $feed, FetcherResultInterface $fetcher_result, StateInterface $state) {

    // Look at our response.
    $rawResponseString = trim($fetcher_result->getRaw());
    if (!strlen($rawResponseString)) {
      throw new EmptyFeedException();
    }

    // Create our result object to add to.
    $result = new ParserResult();

    // Use our ical class to parse the feed.
    try {

      // Create ical parsing object.
      $ical = new ICal('ICal.ics', [
      // Default value.
        'defaultSpan'                 => 2,
        'defaultTimeZone'             => 'UTC',
      // Default value.
        'defaultWeekStart'            => 'MO',
      // Default value.
        'disableCharacterReplacement' => FALSE,
      // Default value.
        'skipRecurrence'              => FALSE,
      // Default value.
        'useTimeZoneWithRRules'       => FALSE,
      ]);
      $ical->initString($rawResponseString);

      // Get and loop over events.
      $events = $ical->events();
      foreach ($events as $eventIndex => $event) {

        // Create item to return for processing.
        $itemObject = new IcalItem();
        // Loop over information from feed and add to item.
        foreach ($event as $eventProperty => $eventPropertyValue) {

          switch ($eventProperty) {
            case 'dtstart':
              $itemObject->set($eventProperty, $event->dtstart_array[2]);
              break;

            case 'dtend':
              if (!empty($event->dtend_array[2])) {
                  $itemObject->set($eventProperty, $event->dtend_array[2]);
                }
              break;

            default:
              $itemObject->set($eventProperty, $eventPropertyValue);

          }

        }

        // Add item to results.
        $result->addItem($itemObject);

      }

    }
    catch (\Exception $e) {
      watchdog_exeption('feeds_ical', $e);
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getMappingSources() {
    return [
      'dtstart' => [
        'label' => $this->t('DTSTART'),
      ],
      'dtend' => [
        'label' => $this->t('DTEND'),
      ],
      'dtstamp' => [
        'label' => $this->t('DTSTAMP'),
      ],
      'rrule' => [
        'label' => $this->t('RRULE'),
      ],
      'uid' => [
        'label' => $this->t('UID'),
        'suggestions' => [
          'targets' => ['guid'],
        ],
      ],
      'created' => [
        'label' => $this->t('CREATED'),
      ],
      'description' => [
        'label' => $this->t('DESCRIPTION'),
      ],
      'lastmodified' => [
        'label' => $this->t('LAST-MODIFIED'),
      ],
      'location' => [
        'label' => $this->t('LOCATION'),
      ],
      'sequence' => [
        'label' => $this->t('SEQUENCE'),
      ],
      'status' => [
        'label' => $this->t('STATUS'),
      ],
      'summary' => [
        'label' => $this->t('SUMMARY'),
        'suggestions' => [
          'targets' => ['title'],
        ],
      ],
      'transp' => [
        'label' => $this->t('TRANSP'),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedCustomSourcePlugins(): array {
    return ['feeds_ical'];
  }

}
